﻿// Snegopat Engine main file
/*
   Основная точка входа. Вызывается в момент, когда snegopat.dll средствами WinAPI
   отловил создание окна заставки 1С, но она еще не показана. Все 1С-ные дллки загружены.
   однако часть функций вызывать еще небезопасно, так как часть инициализации 1С происходит
   после создания основного окна. Поэтому сейчас надо инициализировать то, что нужно и можно
   до создания основного окна, и отлавливать его создание, дабы запустить всё остальное
*/
void main()
{
    initSymbolClassTable();
    initSnegopatProfile();
    setTextHooks();
    initMainFrameWork();
}

// Заполняем таблицу, в которой указан класс символа. Таблица используется в snegopat.dll, но
// для заполнения используются функции V*.
void initSymbolClassTable()
{
    for (uint k = 0; k <= 32; k++)
        pokeByte(symbolClassTable + k, symbSpace);
    for (uint k = 33; k < 0x10000; k++) {
        if (is_space(k))
            pokeByte(symbolClassTable + k, symbSpace);
        else if (is_alpha(k))
            pokeByte(symbolClassTable + k, symbAlpha);
    }
}

// Подключение к дереву настроек файла snegopat.pfl
// Для обеспечения портабельности файл лежит в каталоге снегопата, поэтому
// этот каталог должен быть доступен для записи.
void initSnegopatProfile()
{
    IProfileSource@ snegopatProfileSource;
    currentProcess().createByClsid(CLSID_FileProfileSrc, IID_IProfileSource, snegopatProfileSource);
    snegopatProfileSource.init("file://" + myFolder + "snegopat.pfl");
    getProfileService().attachSource(snegopatProfileSource, pflSnegopat);
}

// Установка различных перехватов, связанных с текстовыми окнами/документами
void setTextHooks()
{
    //debugger();
    // Надо установить перехват на создании окон Text control и text view.
    // Объект TextCtrl можно создать прямо сейчас и установить перехват в его виртуальной таблице
    IWindowView@ wnd;
    currentProcess().createByClsid(CLSID_TxtEdtCtrl, IID_IWindowView, wnd);
    if (wnd is null)
        Print("Не удалось создать TxtEdtCtrl IWindowView");
    else {
        trTxtEdtCtrl_createWnd.setTrap(wnd, IWindowView_createWindow, TxtEdtCtrl_createWindow);
        ICommandTarget@ tgt = cast<IUnknown>(wnd);
        if (tgt is null)
            Print("Не удалось получить ICommandTarget из TxtEdtCtrl IWindowView");
        else // Ставим перехват на получение окном текстового контрола команды
            trTxtCtrCommand_onExecute.setTrap(tgt, ICommandTarget_onExecute, TxtCtrCommand_onExecute);
    }
    // Теперь установим перехват на назначение текстового расширения текстовому документу
    IDocument@ doc;
    currentProcess().createByClsid(CLSID_TxtEdtDoc, IID_IDocument, doc);
    if (doc is null)
        Print("Не удалось создать TxtEdtDoc IDocument");
    else {
        ITextManager_Operations@ to = cast<IUnknown>(doc);
        if (to is null)
            Print("Не удалось получить ITextManager_Operations из TxtEdtDoc");
        else
            trTxtMgr_setExtender.setTrap(to, ITextManager_Operations_setExtenderCLSID, TxtMgrOper_setExtender);
        // Далее устанавливаем перехват на TxtEdtDoc::createView
        // он нужен только один раз, чтобы получить готовый объект TxtEdtView, т.к. создать его сейчас
        // самостоятельно ещё рано, программа вылетит.
        trTxtEdtView_createWnd.setTrap(doc, IDocument_createView, TxtDoc_createView);
    }
}

uint TxtDoc_createView(IDocument& doc, uint pView)
{
    //Print("create text view");
    //debugger();
    trTxtEdtView_createWnd.swap();
    IUnknown@ unk;
    doc.createView(unk);
    IWindowView@ wnd = unk;
    if (wnd is null)
        Print("Не удалось создать TxtEdtView IWindowView");
    else {
        trTxtEdtView_createWnd.setTrap(wnd, IWindowView_createWindow, TxtEdtView_createWindow);
        ICommandTarget@ tgt = cast<IUnknown>(wnd);
        if (tgt is null)
            Print("Не удалось получить ICommandTarget из TxtEdtWnd IWindowView");
        else // Ставим перехват на получение окном текстового контрола команды
            trTxtEdtCommand_onExecute.setTrap(tgt, ICommandTarget_onExecute, TxtEdtCommand_onExecute);
    }
    unk.AddRef();
    uint unkAddr = getAddress(unk);
    pokeDword(pView, unkAddr);
    return pView;
}

TrapVirtualStdCall trTxtEdtView_createWnd;
TrapVirtualStdCall trTxtEdtCommand_onExecute;
TrapVirtualStdCall trTxtEdtCtrl_createWnd;
TrapVirtualStdCall trTxtCtrCommand_onExecute;
TrapVirtualStdCall trTxtMgr_setExtender;

bool TxtEdtCtrl_createWindow(IWindowView& view, HWND parentWindow)
{
    //debugger();
    trTxtEdtCtrl_createWnd.swap();
    bool res = view.createWindow(parentWindow);
    trTxtEdtCtrl_createWnd.swap();
    onCreateTextWnd(view, true);
    /*
    if (res)
        Print("Create text ctrl hwnd=" + view.hwnd());
    else
        Print("Fail create text ctrl");
    */
    return res;
}

bool TxtEdtView_createWindow(IWindowView& view, HWND parentWindow)
{
    //debugger();
    trTxtEdtView_createWnd.swap();
    bool res = view.createWindow(parentWindow);
    trTxtEdtView_createWnd.swap();
    onCreateTextWnd(view, false);
    /*
    if (res)
		Message("create text view in 0x" + formatInt(parentWindow, "0X", 8) + " hwnd=0x" + formatInt(view.hwnd(), "0X", 8), mmInfo);
    else
        Message("Fail create text view", mmExc3);
    */
    return res;
}

bool checkCommandAndHideSmartBox(ICommandTarget& tgt, const Command& command, TrapVirtualStdCall& trap)
{
    //Message("Txt View Command execute " + command.id.group + " num=" + command.id.num + " param=" + command.param);
    /*
    if (IntelliSite::get().active()) {
    unsigned int cs = commandState(command.id);
    if (cs & cmdStateEnabled)
    IntelliSite::get().hide();
    }*/
    trap.swap();
    bool res = tgt.onExecute(command);
    trap.swap();
    return res;
}

bool TxtEdtCommand_onExecute(ICommandTarget& tgt, const Command& command)
{
    return checkCommandAndHideSmartBox(tgt, command, trTxtEdtCommand_onExecute);
}

bool TxtCtrCommand_onExecute(ICommandTarget& tgt, const Command& command)
{
    return checkCommandAndHideSmartBox(tgt, command, trTxtCtrCommand_onExecute);
}

int TxtMgrOper_setExtender(ITextManager_Operations& to, const Guid& clsid)
{
    //Print("TxtMgrOper_setExtender " + clsid);
    trTxtMgr_setExtender.swap();
    int res = to.setExtenderCLSID(clsid);
    trTxtMgr_setExtender.swap();

    /*
    ITextManagerPtr txt = (IUnknown*)this;
    ModuleInfo* pModInfo = ModuleInfo::findByITextManager(txt);
    if (pModInfo)
        pModInfo->registerExtender(clsid);
    */
    return res;
}
