﻿// textwnd.as
/*
  Тут часть кода, работающая с текстовыми окнами
*/
TextWnd@ activeTextWnd;

// Это интерфейс обработчика событий в тексте.
// В-зависимости от назначенного текстовому документу расширения (встроенный язык, язык запросов и т.п.)
// создается соответствующий расширению текстовый процессор. Именно он и обрабатывает события в текстовом окне
interface TextProcessor
{
};

TextDocStorage textDocStorage;

class TextDocStorage
{
    array<TextDoc@> openedDocs;

    TextDoc@ find(ITextManager@ itm)
    {
        for (uint idx = 0, size = openedDocs.length(); idx < size; idx++) {
            if (openedDocs[idx].itm is itm)
                return openedDocs[idx];
        }
        return null;
    }
    TextDoc@ find(TextManager@ tm)
    {
        for (uint idx = 0, size = openedDocs.length(); idx < size; idx++) {
            if (openedDocs[idx].tm is tm)
                return openedDocs[idx];
        }
        return null;
    }
    void store(TextDoc@ tdoc) { openedDocs.insertLast(tdoc); }
    void remove(TextDoc@ doc)
    {
        for (uint idx = 0, size = openedDocs.length(); idx < size; idx++) {
            if (openedDocs[idx] is doc) {
                openedDocs.removeAt(idx);
                break;
            }
        }
    }
};

void onCreateTextWnd(IWindowView@ pWnd, bool bControl)
{
    debugger();
    if (bControl) {
        IControlDesign@ ctrl = cast<IUnknown>(pWnd);
        if (ctrl !is null && ctrl.getMode() != ctrlRunning)
            return; // С контролами на формах не работаем
    }
    TextWnd wnd(pWnd, bControl);
    ITextEditor@ editor = cast<IUnknown>(pWnd);
    ITextManager@ itm;
    editor.getITextManager(itm);
    TextDoc@ tdoc = textDocStorage.find(itm);
    if (tdoc is null) {
        //Print("Text doc not found");
        @tdoc = TextDoc();
        @tdoc.itm = itm;
        @tdoc.tm = itm.getTextManager();
        textDocStorage.store(tdoc);
    } //else Print("Found old text doc");
    tdoc.attachView(wnd);
}

// Класс, который хранит различную инфу о текстовом документе.
// Важно, у одного текстового документа может быть открыто несколько окон
class TextDoc
{
    ITextManager@   itm;
    TextManager@    tm;
    array<TextWnd@> views;

    void attachView(TextWnd@ wnd)
    {
        @wnd.textDoc = this;
        views.insertLast(@wnd);
    }
    void detachView(TextWnd@ wnd)
    {
        if (wnd is activeTextWnd)
            @activeTextWnd = null;
        for (uint idx = 0, size = views.length(); idx < size; idx++) {
            if (views[idx] is wnd) {
                views.removeAt(idx);
                break;
            }
        }
        if (views.length() == 0) { // Закрыли последнее окно
            textDocStorage.remove(this);
            //Message("Close last wnd");
        }
    }
};

class TextWnd
{
    ASWnd@ wnd;
    HWND hWnd;
    IWindowView@ view;
    ITextEditor@ ted;
    TextDoc@ textDoc;
    bool isControl;
    
    TextWnd(IWindowView@ v, bool isCtrl)
    {
        isControl = isCtrl;
        @view = v;
        hWnd = view.hwnd();
        @wnd = attachWndToFunction(hWnd, WndFunc(this.WndProc),
                    array<uint> = { WM_KEYDOWN, WM_CHAR, WM_DESTROY, WM_SETFOCUS, WM_KILLFOCUS });
        @ted = cast<IUnknown>(view);
    }
    uint WndProc(uint msg, uint w, uint l)
    {
        switch (msg) {
        case WM_SETFOCUS:
            @activeTextWnd = this;
            break;
        case WM_KILLFOCUS:
            @activeTextWnd = null;
            break;
        case WM_DESTROY:
            if (textDoc !is null)
                textDoc.detachView(this);
            break;
        case WM_KEYDOWN:
            return onKeyDown(w, l);
        case WM_CHAR:
            return onChar(w);
        }
        return wnd.doDefault();
    }
    ~TextWnd()
    {
        //Message("destroy TextWnd");
    }
    uint onKeyDown(uint w, uint l)
    {
        return wnd.doDefault();
    }
    uint onChar(uint16 symb)
    {
        // сначала дадим отработать штатным механизмам
        uint res = wnd.doDefault();

        // а затем уже мы подключимся
        //string t = " "; t[0] = symb; Message("WM_CHAR " + uint(symb) + " = " + t);
        if (this !is activeTextWnd)
            return res;		// Не в фокусе не работаем, возникает при наборе в штатной подсказке
        if (ted !is null) {
            TextPosition tp;
            ted.getCaretPosition(tp, false);
            //Message("Caret line=" + tp.line + " col=" + tp.col);
            v8string line;
            textDoc.tm.getLine(tp.line, line);
            if (is_alnum(symb)) {
                showAssist();
            }
            //Message(line);

        }
        return res;
    }
};

void showAssist()
{
    ICommandReceiver@ pCmdRcv = cast<IUnknown>(coreMainFrame);
    if (pCmdRcv !is null) {
        CommandID cmd;
        cmd.group = cmdFrameGroup;
        cmd.num = cmdFrameShowAssist;
        if (pCmdRcv.hasHandler(cmd))
            pCmdRcv.transmitCommand(Command(cmd, 0));
    }
}
